@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2016, University of Amsterdam
+    Copyright (c)  1997-2018, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -85,8 +85,8 @@ translate_message2(Term) -->
     prolog:message(Term).
 translate_message2(Term) -->
     prolog_message(Term).
-translate_message2(error(resource_error(stack), Name)) -->
-    [ 'Out of ~w stack'-[Name] ].
+translate_message2(error(resource_error(stack), Context)) -->
+    out_of_stack(Context).
 translate_message2(error(resource_error(Missing), _)) -->
     [ 'Not enough resources: ~w'-[Missing] ].
 translate_message2(error(ISO, SWI)) -->
@@ -381,6 +381,16 @@ swi_message(conditional_compilation_error(unterminated, Where)) -->
     cond_location(Where).
 swi_message(conditional_compilation_error(no_if, What)) -->
     [ ':- ~w without :- if'-[What] ].
+swi_message(duplicate_key(Key)) -->
+    [ 'Duplicate key: ~p'-[Key] ].
+swi_message(initialization_error(failed, Goal, File:Line)) -->
+    !,
+    [ '~w:~w: ~p: false'-[File, Line, Goal] ].
+swi_message(initialization_error(Error, Goal, File:Line)) -->
+    [ '~w:~w: ~p '-[File, Line, Goal] ],
+    translate_message(Error).
+swi_message(qlf_format_error(File, Message)) -->
+    [ '~w: Invalid QLF file: ~w'-[File, Message] ].
 
 cond_location(File:Line) -->
     { file_base_name(File, Base) },
@@ -464,9 +474,9 @@ swi_comment(Msg) -->
 
 
 thread_context -->
-    { thread_self(Me), Me \== main },
+    { thread_self(Me), Me \== main, thread_property(Me, id(Id)) },
     !,
-    ['[Thread ~w] '-[Me]].
+    ['[Thread ~w] '-[Id]].
 thread_context -->
     [].
 
@@ -494,6 +504,24 @@ prolog_message(initialization_failure(Goal, _)) -->
 prolog_message(initialization_exception(E)) -->
     [ 'Prolog initialisation failed:', nl ],
     translate_message(E).
+prolog_message(init_goal_syntax(Error, Text)) -->
+    !,
+    [ '-g ~w: '-[Text] ],
+    translate_message(Error).
+prolog_message(init_goal_failed(failed, @(Goal,File:Line))) -->
+    !,
+    [ '~w:~w: ~p: false'-[File, Line, Goal] ].
+prolog_message(init_goal_failed(Error, @(Goal,File:Line))) -->
+    !,
+    [ '~w:~w: ~p '-[File, Line, Goal] ],
+    translate_message(Error).
+prolog_message(init_goal_failed(failed, Text)) -->
+    !,
+    [ '-g ~w: false'-[Text] ].
+prolog_message(init_goal_failed(Error, Text)) -->
+    !,
+    [ '-g ~w: '-[Text] ],
+    translate_message(Error).
 prolog_message(unhandled_exception(E)) -->
     [ 'Unhandled exception: ' ],
     (   translate_message2(E)
@@ -543,6 +571,8 @@ prolog_message(unknown_in_module_user) -->
       'Please use :- dynamic or limit usage of unknown to a module.', nl,
       'See http://www.swi-prolog.org/howto/database.html'
     ].
+prolog_message(deprecated(What)) -->
+    deprecated(What).
 
 
                  /*******************************
@@ -627,6 +657,13 @@ prolog_message(minus_in_identifier) -->
     ].
 prolog_message(qlf(removed_after_error(File))) -->
     [ 'Removed incomplete QLF file ~w'-[File] ].
+prolog_message(qlf(recompile(Spec,_Pl,_Qlf,Reason))) -->
+    [ '~p: recompiling QLF file'-[Spec] ],
+    qlf_recompile_reason(Reason).
+prolog_message(qlf(can_not_recompile(Spec,QlfFile,_Reason))) -->
+    [ '~p: can not recompile "~w" (access denied)'-[Spec, QlfFile], nl,
+      '\tLoading from source'-[]
+    ].
 prolog_message(redefine_module(Module, OldFile, File)) -->
     [ 'Module "~q" already loaded from ~w.'-[Module, OldFile], nl,
       'Wipe and reload from ~w? '-[File], flush
@@ -706,6 +743,12 @@ pi_head(Name/Arity, user:Head) :-
     atom(Name), integer(Arity),
     functor(Head, Name, Arity).
 
+qlf_recompile_reason(old) -->
+    !,
+    [ ' (out of date)'-[] ].
+qlf_recompile_reason(_) -->
+    [ ' (incompatible with current Prolog version)'-[] ].
+
 prolog_message(file_search(cache(Spec, _Cond), Path)) -->
     [ 'File search: ~p --> ~p (cache)'-[Spec, Path] ].
 prolog_message(file_search(found(Spec, Cond), Path)) -->
@@ -755,6 +798,120 @@ prolog_message(cgc(done(CollectedClauses, _CollectedBytes,
       [CollectedClauses, Time, RemainingBytes]
     ].
 
+		 /*******************************
+		 *        STACK OVERFLOW	*
+		 *******************************/
+
+out_of_stack(Context) -->
+    { human_stack_size(Context.localused,  Local),
+      human_stack_size(Context.globalused, Global),
+      human_stack_size(Context.trailused,  Trail),
+      current_prolog_flag(stack_limit, LimitBytes),
+      LimitK is LimitBytes//1024,
+      human_stack_size(LimitK, Limit),
+      LCO is (100*(Context.depth - Context.environments))/Context.depth
+    },
+    [ 'Stack limit (~s) exceeded'-[Limit], nl,
+      '  Stack sizes: local: ~s, global: ~s, trail: ~s'-[Local,Global,Trail], nl,
+      '  Stack depth: ~D, last-call: ~0f%, Choice points: ~D'-
+         [Context.depth, LCO, Context.choicepoints], nl
+    ],
+    overflow_reason(Context, Resolve),
+    resolve_overflow(Resolve).
+
+human_stack_size(Size, String) :-
+    Size < 100,
+    format(string(String), '~dKb', [Size]).
+human_stack_size(Size, String) :-
+    Size < 100 000,
+    Value is Size / 1024,
+    format(string(String), '~1fMb', [Value]).
+human_stack_size(Size, String) :-
+    Value is Size / (1024*1024),
+    format(string(String), '~1fGb', [Value]).
+
+overflow_reason(Context, fix) -->
+    show_non_termination(Context),
+    !.
+overflow_reason(Context, enlarge) -->
+    { Stack = Context.get(stack) },
+    !,
+    [ '  In:'-[], nl ],
+    stack(Stack).
+overflow_reason(_Context, enlarge) -->
+    [ '  Insufficient global stack'-[] ].
+
+show_non_termination(Context) -->
+    (   { Stack = Context.get(cycle) }
+    ->  [ '  Probable infinite recursion (cycle):'-[], nl ]
+    ;   { Stack = Context.get(non_terminating) }
+    ->  [ '  Possible non-terminating recursion:'-[], nl ]
+    ),
+    stack(Stack).
+
+stack([]) --> [].
+stack([frame(Depth, M:Goal, _)|T]) -->
+    [ '    [~D] ~q:'-[Depth, M] ],
+    stack_goal(Goal),
+    [ nl ],
+    stack(T).
+
+stack_goal(Goal) -->
+    { compound(Goal),
+      !,
+      compound_name_arity(Goal, Name, Arity)
+    },
+    [ '~q('-[Name] ],
+    stack_goal_args(1, Arity, Goal),
+    [ ')'-[] ].
+stack_goal(Goal) -->
+    [ '~q'-[Goal] ].
+
+stack_goal_args(I, Arity, Goal) -->
+    { I =< Arity,
+      !,
+      arg(I, Goal, A),
+      I2 is I + 1
+    },
+    stack_goal_arg(A),
+    (   { I2 =< Arity }
+    ->  [ ', '-[] ],
+        stack_goal_args(I2, Arity, Goal)
+    ;   []
+    ).
+stack_goal_args(_, _, _) -->
+    [].
+
+stack_goal_arg(A) -->
+    { nonvar(A),
+      A = [Len|T],
+      !
+    },
+    (   {Len == cyclic_term}
+    ->  [ '[cyclic list]'-[] ]
+    ;   {T == []}
+    ->  [ '[length:~D]'-[Len] ]
+    ;   [ '[length:~D|~p]'-[Len, T] ]
+    ).
+stack_goal_arg(A) -->
+    { nonvar(A),
+      A = _/_,
+      !
+    },
+    [ '<compound ~p>'-[A] ].
+stack_goal_arg(A) -->
+    [ '~p'-[A] ].
+
+resolve_overflow(fix) -->
+    [].
+resolve_overflow(enlarge) -->
+    { current_prolog_flag(stack_limit, LimitBytes),
+      NewLimit is LimitBytes * 2
+    },
+    [ nl,
+      'Use the --stack_limit=size[KMG] command line option or'-[], nl,
+      '?- set_prolog_flag(stack_limit, ~I). to double the limit.'-[NewLimit]
+    ].
 
 
                  /*******************************
@@ -834,10 +991,9 @@ compiler_warning(neq_singleton(A,B), _Clause, Options) -->
     [ 'Test is always true: ~W'-[A\==B, Options] ].
 compiler_warning(unify_singleton(A,B), _Clause, Options) -->
     [ 'Unified variable is not used: ~W'-[A=B, Options] ].
-compiler_warning(var_true(A), _Clause, Options) -->
-    [ 'Test is always true: ~W'-[var(A), Options] ].
-compiler_warning(nonvar_false(A), _Clause, Options) -->
-    [ 'Test is always false: ~W'-[nonvar(A), Options] ].
+compiler_warning(always(Bool, Pred, Arg), _Clause, Options) -->
+    { Goal =.. [Pred,Arg] },
+    [ 'Test is always ~w: ~W'-[Bool, Goal, Options] ].
 compiler_warning(unbalanced_var(V), _Clause, Options) -->
     [ 'Variable not introduced in all branches: ~W'-[V, Options] ].
 compiler_warning(branch_singleton(V), _Clause, Options) -->
@@ -862,12 +1018,12 @@ prolog_message(version) -->
     !,
     [ '~w'-[Version] ].
 prolog_message(version) -->
-    { current_prolog_flag(version, Version),
-      Major is Version // 10000,
-      Minor is (Version // 100) mod 100,
-      Patch is Version mod 100
+    { current_prolog_flag(version_data, swi(Major,Minor,Patch,Options))
     },
-    [ '~w.~w.~w'-[Major, Minor, Patch] ].
+    (   { memberchk(tag(Tag), Options) }
+    ->  [ '~w.~w.~w-~w'-[Major, Minor, Patch, Tag] ]
+    ;   [ '~w.~w.~w'-[Major, Minor, Patch] ]
+    ).
 prolog_message(address_bits) -->
     { current_prolog_flag(address_bits, Bits)
     },
@@ -1313,6 +1469,8 @@ prolog_message(null_byte_in_path(Component)) -->
 prolog_message(invalid_tmp_var(Var, Value, Reason)) -->
     [ 'Cannot use '-[] ], env(Var),
     [ ' as temporary file directory: ~p: ~w'-[Value, Reason] ].
+prolog_message(ambiguous_stream_pair(Pair)) -->
+    [ 'Ambiguous operation on stream pair ~p'-[Pair] ].
 
 env(Name) -->
     { current_prolog_flag(windows, true) },
@@ -1320,6 +1478,16 @@ env(Name) -->
 env(Name) -->
     [ '$~w'-[Name] ].
 
+		 /*******************************
+		 *          DEPRECATED		*
+		 *******************************/
+
+deprecated(set_prolog_stack(_Stack,limit)) -->
+    [ 'set_prolog_stack/2: limit(Size) sets the combined limit.'-[], nl,
+      'See http://www.swi-prolog.org/changes/stack-limit.html'
+    ].
+
+
                  /*******************************
                  *      PRINTING MESSAGES       *
                  *******************************/
@@ -1582,6 +1750,7 @@ append_args(Args0, Args1, Args) :-
 
 print_once(compatibility(_), _).
 print_once(null_byte_in_path(_), _).
+print_once(deprecated(_), _).
 
 %!  must_print(+Level, +Message)
 %
