@@ -105,17 +105,18 @@ clause_info(ClauseRef, File, TermPos, NameOffset, Options) :-
     ;   true
     ),
     clause_property(ClauseRef, file(File)),
-    '$clause'(Head, Body, ClauseRef, VarOffset),
+    File \== user,                  % loaded using ?- [user].
+    '$clause'(Head0, Body, ClauseRef, VarOffset),
+    (   module_property(Module, file(File))
+    ->  true
+    ;   strip_module(user:Head0, Module, _)
+    ),
+    unqualify(Head0, Module, Head),
     (   Body == true
     ->  DecompiledClause = Head
     ;   DecompiledClause = (Head :- Body)
     ),
-    File \== user,                  % loaded using ?- [user].
     clause_property(ClauseRef, line_count(LineNo)),
-    (   module_property(Module, file(File))
-    ->  true
-    ;   strip_module(user:Head, Module, _)
-    ),
     debug(clause_info, 'from ~w:~d ... ', [File, LineNo]),
     read_term_at_line(File, LineNo, Module, Clause, TermPos0, VarNames),
     option(variable_names(VarNames), Options, _),
@@ -126,6 +127,11 @@ clause_info(ClauseRef, File, TermPos, NameOffset, Options) :-
     debug(clause_info, 'got names~n', []),
     !.
 
+unqualify(Module:Head, Module, Head) :-
+    !.
+unqualify(Head, _, Head).
+
+
 %!  unify_term(+T1, +T2)
 %
 %   Unify the two terms, where T2 is created by writing the term and
@@ -444,6 +450,9 @@ ubody(B, C, _, P, P) :-
     B =@= C, B = C,
     does_not_dcg_after_binding(B, P),
     !.
+ubody(X0, X, M, parentheses_term_position(_, _, P0), P) :-
+    !,
+    ubody(X0, X, M, P0, P).
 ubody(X, call(X), _,                    % X = call(X)
       Pos,
       term_position(From, To, From, To, [Pos])) :-
@@ -541,6 +550,10 @@ conj((A,B), brace_term_position(_,T,PA), GL, TG, PL, TP) :-
     conj(A, PA, GL, TGA, PL, TPA),
     T1 is T - 1,
     conj(B, T1-T, TGA, TG, TPA, TP).
+conj(A, parentheses_term_position(_,_,Pos), GL, TG, PL, TP) :-
+    nonvar(Pos),
+    !,
+    conj(A, Pos, GL, TG, PL, TP).
 conj((!,(S=SR)), F-T, [!,S=SR|TG], TG, [F-T,F1-T1|TP], TP) :-
     F1 is F+1,
     T1 is T+1.
@@ -742,21 +755,21 @@ goal_expansion(send_super(R, Msg), send_class(R, _, Msg), P, P).
 goal_expansion(get_super(R, Msg, V), get_class(R, _, Msg, V), P, P).
 goal_expansion(SendSuperN, send_class(R, _, Msg), P, P) :-
     compound(SendSuperN),
-    SendSuperN =.. [send_super, R, Sel | Args],
+    compound_name_arguments(SendSuperN, send_super, [R,Sel|Args]),
     Msg =.. [Sel|Args].
 goal_expansion(SendN, send(R, Msg), P, P) :-
     compound(SendN),
-    SendN =.. [send, R, Sel | Args],
+    compound_name_arguments(SendN, send, [R,Sel|Args]),
     atom(Sel), Args \== [],
     Msg =.. [Sel|Args].
 goal_expansion(GetSuperN, get_class(R, _, Msg, Answer), P, P) :-
     compound(GetSuperN),
-    GetSuperN =.. [get_super, R, Sel | AllArgs],
+    compound_name_arguments(GetSuperN, get_super, [R,Sel|AllArgs]),
     append(Args, [Answer], AllArgs),
     Msg =.. [Sel|Args].
 goal_expansion(GetN, get(R, Msg, Answer), P, P) :-
     compound(GetN),
-    GetN =.. [get, R, Sel | AllArgs],
+    compound_name_arguments(GetN, get, [R,Sel|AllArgs]),
     append(Args, [Answer], AllArgs),
     atom(Sel), Args \== [],
     Msg =.. [Sel|Args].
