@@ -643,6 +643,12 @@ PRED_IMPL("time_file", 2, time_file, 0)
 
   if ( PL_get_file_name(A1, &fn, 0) )
   { double time;
+    int sl;
+
+    if ( (sl=file_name_is_iri(fn)) )
+    { return ( iri_hook(fn, IRI_TIME, &time) &&
+	       PL_unify_float(A2, time) );
+    }
 
     if ( LastModifiedFile(fn, &time) )
       return PL_unify_float(A2, time);
@@ -662,6 +668,12 @@ PRED_IMPL("size_file", 2, size_file, 0)
 
   if ( PL_get_file_name(A1, &n, 0) )
   { int64_t size;
+    int sl;
+
+    if ( (sl=file_name_is_iri(n)) )
+    { return ( iri_hook(n, IRI_SIZE, &size) &&
+	       PL_unify_int64(A2, size) );
+    }
 
     if ( (size = SizeFile(n)) < 0 )
       return PL_error("size_file", 2, OsError(), ERR_FILE_OPERATION,
@@ -680,6 +692,7 @@ PRED_IMPL("access_file", 2, access_file, 0)
   char *n;
   int md;
   atom_t m;
+  int sl;
 
   term_t name = A1;
   term_t mode = A2;
@@ -703,13 +716,24 @@ PRED_IMPL("access_file", 2, access_file, 0)
   else
     return PL_error("access_file", 2, NULL, ERR_DOMAIN, ATOM_io_mode, mode);
 
+  if ( (sl=file_name_is_iri(n)) )
+  { int rc;
+
+    if ( !iri_hook(n, IRI_ACCESS, md, &rc) )
+      return FALSE;
+
+    return rc;
+  }
+
   if ( AccessFile(n, md) )
     return TRUE;
 
   if ( md == ACCESS_WRITE && !AccessFile(n, ACCESS_EXIST) )
   { char tmp[MAXPATHLEN];
-    char *dir = DirName(n, tmp);
+    char *dir;
 
+    if ( !(dir = DirName(n, tmp)) )
+      return PL_representation_error("max_path_length");
     if ( dir[0] )
     { if ( !ExistsDirectory(dir) )
 	return FALSE;
@@ -735,9 +759,9 @@ PRED_IMPL("read_link", 3, read_link, 0)
     return FALSE;
 
   if ( (l = ReadLink(n, buf)) &&
-       PL_unify_atom_chars(link, l) &&
+       PL_unify_chars(link, PL_ATOM|REP_FN, (size_t)-1, l) &&
        (t = DeRefLink(n, buf)) &&
-       PL_unify_atom_chars(to, t) )
+       PL_unify_chars(to, PL_ATOM|REP_FN, (size_t)-1, t) )
     return TRUE;
 
   return FALSE;
@@ -747,10 +771,20 @@ PRED_IMPL("read_link", 3, read_link, 0)
 static
 PRED_IMPL("exists_file", 1, exists_file, 0)
 { char *n;
+  int sl;
 
   if ( !PL_get_file_name(A1, &n, 0) )
     return FALSE;
 
+  if ( (sl=file_name_is_iri(n)) )
+  { int rc;
+
+    if ( !iri_hook(n, IRI_ACCESS, ACCESS_FILE, &rc) )
+      return FALSE;
+
+    return rc;
+  }
+
   return ExistsFile(n);
 }
 
@@ -758,10 +792,20 @@ PRED_IMPL("exists_file", 1, exists_file, 0)
 static
 PRED_IMPL("exists_directory", 1, exists_directory, 0)
 { char *n;
+  int sl;
 
   if ( !PL_get_file_name(A1, &n, 0) )
     return FALSE;
 
+  if ( (sl=file_name_is_iri(n)) )
+  { int rc;
+
+    if ( !iri_hook(n, IRI_ACCESS, ACCESS_DIRECTORY, &rc) )
+      return FALSE;
+
+    return rc;
+  }
+
   return ExistsDirectory(n);
 }
 
@@ -771,7 +815,7 @@ PRED_IMPL("is_absolute_file_name", 1, is_absolute_file_name, 0)
 { char *n;
 
   if ( PL_get_file_name(A1, &n, 0) &&
-       IsAbsolutePath(n) )
+       (IsAbsolutePath(n) || file_name_is_iri(n)) )
     return TRUE;
 
   return FALSE;
@@ -792,24 +836,31 @@ PRED_IMPL("same_file", 2, same_file, 0)
 
 static
 PRED_IMPL("file_base_name", 2, file_base_name, 0)
-{ char *n;
-
-  if ( !PL_get_chars(A1, &n, CVT_ALL|REP_FN|CVT_EXCEPTION) )
-    return FALSE;
-
-  return PL_unify_chars(A2, PL_ATOM|REP_FN, -1, BaseName(n));
-}
-
-
-static
-PRED_IMPL("file_directory_name", 2, file_directory_name, 0)
-{ char *n;
+{ char *n, *b;
   char tmp[MAXPATHLEN];
 
   if ( !PL_get_chars(A1, &n, CVT_ALL|REP_FN|CVT_EXCEPTION) )
     return FALSE;
 
-  return PL_unify_chars(A2, PL_ATOM|REP_FN, -1, DirName(n, tmp));
+  if ( (b=BaseName(n, tmp)) )
+    return PL_unify_chars(A2, PL_ATOM|REP_FN, -1, b);
+  else
+    return PL_representation_error("max_path_length");
+}
+
+
+static
+PRED_IMPL("file_directory_name", 2, file_directory_name, 0)
+{ char *n, *d;
+  char tmp[MAXPATHLEN];
+
+  if ( !PL_get_chars(A1, &n, CVT_ALL|REP_FN|CVT_EXCEPTION) )
+    return FALSE;
+
+  if ( (d=DirName(n, tmp)) )
+    return PL_unify_chars(A2, PL_ATOM|REP_FN, -1, d);
+  else
+    return PL_representation_error("max_path_length");
 }
 
 
@@ -976,9 +1027,10 @@ PRED_IMPL("$absolute_file_name", 2, absolute_file_name, 0)
   term_t name = A1;
   term_t expanded = A2;
 
-  if ( PL_get_file_name(name, &n, 0) &&
-       (n = AbsoluteFile(n, tmp)) )
-    return PL_unify_chars(expanded, PL_ATOM|REP_FN, -1, n);
+  if ( PL_get_file_name(name, &n, 0) )
+  { if ( (n = AbsoluteFile(n, tmp)) )
+      return PL_unify_chars(expanded, PL_ATOM|REP_FN, -1, n);
+  }
 
   return FALSE;
 }
