@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2017, University of Amsterdam
+    Copyright (c)  2011-2018, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -864,7 +864,7 @@ OsPath(const char *p, char *buf)
     This  function  is  based  on  a  similar  (primitive)  function  in
     Edinburgh C-Prolog.
 
-    char *BaseName(path)
+    char *BaseName(path, char *base)
 	 char *path;
 
     Return the basic file name for a file having path `path'.
@@ -1133,6 +1133,7 @@ char *
 canonicaliseFileName(char *path)
 { char *out = path, *in = path, *start = path;
   tmp_buffer saveb;
+  int sl;
 
 #ifdef O_HASDRIVES			/* C: */
   if ( in[1] == ':' && isLetter(in[0]) )
@@ -1152,6 +1153,11 @@ canonicaliseFileName(char *path)
 #endif
 #endif
 
+  if ( (sl=file_name_is_iri(in)) )
+  { in += (sl+3);
+    out = start = in;
+  }
+
 #ifdef O_HASSHARES			/* //host/ */
   if ( in[0] == '/' && in[1] == '/' && isAlpha(in[2]) )
   { char *s;
@@ -1492,6 +1498,8 @@ IsAbsolutePath(const char *p)				/* /d:/ */
        (p[0] == '\\' && p[1] == '\\') )	/* \\host\share */
     succeed;
 #endif
+  if ( file_name_is_iri(p) )
+    succeed;
 
   fail;
 }
@@ -1529,7 +1537,8 @@ GetCurrentDriveLetter()
 
 int
 IsAbsolutePath(const char *p)
-{ return p[0] == '/';
+{ return ( p[0] == '/' ||
+	   file_name_is_iri(p) );
 }
 
 #endif /*O_HASDRIVES*/
@@ -1665,16 +1674,32 @@ PL_cwd(char *cwd, size_t cwdlen)
 
 
 char *
-BaseName(const char *f)
+BaseName(const char *f, char *base)
 { if ( f )
-  { const char *base;
+  { char *e = (char*)f+strlen(f);
+    char *end;
 
-    for(base = f; *f; f++)
-    { if (*f == '/')
-	base = f+1;
+    if ( e == f )
+    { base[0] = EOS;
+    } else
+    { while(e>f && e[-1] == '/')
+	e--;
+      end = e;
+      while(e>f && e[-1] != '/')
+	e--;
+
+      if ( e == end && *e == '/' )
+      { strcpy(base, "/");
+      } else if ( end-e+1 <= MAXPATHLEN )
+      { memmove(base, e, end-e);		/* may overlap */
+	base[end-e] = EOS;
+      } else
+      { errno = ENAMETOOLONG;
+	return NULL;
+      }
     }
 
-    return (char *)base;
+    return base;
   }
 
   return NULL;
@@ -1684,21 +1709,35 @@ BaseName(const char *f)
 char *
 DirName(const char *f, char *dir)
 { if ( f )
-  { const char *base, *p;
+  { char *e = (char*)f+strlen(f);
 
-    for(base = p = f; *p; p++)
-    { if ( *p == '/' )
-	base = p;
-    }
-    if ( base == f )
-    { if ( *f == '/' )
-	strcpy(dir, "/");
-      else
-	strcpy(dir, ".");
+    if ( e == f )
+    { strcpy(dir, ".");
     } else
-    { if ( dir != f )			/* otherwise it is in-place */
-	strncpy(dir, f, base-f);
-      dir[base-f] = EOS;
+    { while(e>f && e[-1] == '/')
+	e--;
+      while(e>f && e[-1] != '/')
+	e--;
+      while(e>f && e[-1] == '/')
+	e--;
+
+      if ( e == f )
+      { if ( *f == '/' )
+	  strcpy(dir, "/");
+	else
+	  strcpy(dir, ".");
+      } else
+      { if ( dir != f )			/* otherwise it is in-place */
+	{ if ( e-f+1 <= MAXPATHLEN )
+	  { strncpy(dir, f, e-f);
+	    dir[e-f] = EOS;
+	  } else
+	  { errno = ENAMETOOLONG;
+	    return NULL;
+	  }
+	} else
+	  e[0] = EOS;
+      }
     }
 
     return dir;
@@ -2468,9 +2507,11 @@ System(char *cmd)
   if ( (pid = fork()) == -1 )
   { return PL_error("shell", 2, OsError(), ERR_SYSCALL, "fork");
   } else if ( pid == 0 )		/* The child */
-  { Setenv("PROLOGCHILD", "yes");
+  { char tmp[MAXPATHLEN];
+
+    Setenv("PROLOGCHILD", "yes");
     PL_cleanup_fork();
-    execl(shell, BaseName(shell), "-c", cmd, (char *)0);
+    execl(shell, BaseName(shell, tmp), "-c", cmd, (char *)0);
     fatalError("Failed to execute %s: %s", shell, OsError());
     fail;
     /*NOTREACHED*/
