@@ -810,6 +810,7 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
       if ( !PL_get_int64_ex(value, &i) )
 	return FALSE;
       f->value.i = i;
+
 #ifdef O_ATOMGC
       if ( k == ATOM_agc_margin )
 	GD->atoms.margin = (size_t)i;
@@ -817,6 +818,10 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
 #endif
       if ( k == ATOM_table_space )
 	LD->tabling.node_pool.limit = (size_t)i;
+      else if ( k == ATOM_stack_limit )
+      { if ( !set_stack_limit((size_t)i) )
+	  return FALSE;
+      }
       break;
     }
     case FT_FLOAT:
@@ -1045,6 +1050,8 @@ unify_prolog_flag_value(Module m, atom_t key, prolog_flag *f, term_t val)
     return FALSE;
   } else if ( key == ATOM_access_level )
   { return PL_unify_atom(val, accessLevel());
+  } else if ( key == ATOM_stack_limit )
+  { return PL_unify_int64(val, LD->stacks.limit);
   }
 
   switch(f->flags & FT_MASK)
@@ -1317,6 +1324,7 @@ initPrologFlags(void)
   setPrologFlag("agc_margin",FT_INTEGER,	       GD->atoms.margin);
 #endif
   setPrologFlag("table_space", FT_INTEGER, LD->tabling.node_pool.limit);
+  setPrologFlag("stack_limit", FT_INTEGER, LD->stacks.limit);
 #if defined(HAVE_DLOPEN) || defined(HAVE_SHL_LOAD) || defined(EMULATE_DLOPEN)
   setPrologFlag("open_shared_object",	  FT_BOOL|FF_READONLY, TRUE, 0);
   setPrologFlag("shared_object_extension",	  FT_ATOM|FF_READONLY, SO_EXT);
