@@ -3,8 +3,9 @@
     Author:        Keri Harris
     E-mail:        keri.harris@securitease.com
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2016, University of Amsterdam
+    Copyright (c)  2011-2018, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -133,6 +134,7 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_GC_HOLE),
   DEBUG_TOPIC(MSG_GC_SWEEP),
   DEBUG_TOPIC(MSG_GC_CHECK),
+  DEBUG_TOPIC(MSG_SHIFT),
   DEBUG_TOPIC(MSG_SHIFT_PROGRESS),
   DEBUG_TOPIC(MSG_SHIFT_POINTER),
   DEBUG_TOPIC(MSG_SHIFT_FRAME),
@@ -168,6 +170,9 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_TRIE_PUT_TERM),
   DEBUG_TOPIC(MSG_TRIE_GC),
 
+  DEBUG_TOPIC(MSG_ZIP),
+  DEBUG_TOPIC(MSG_ZIP_STATE),
+
   DEBUG_TOPIC(MSG_TABLING_WORK),
 
   DEBUG_TOPIC(CHK_SECURE),
