@@ -3,7 +3,8 @@
     Author:        Keri Harris
     E-mail:        keri.harris@securitease.com
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2016, University of Amsterdam
+    Copyright (c)  2011-2018, University of Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -133,10 +134,11 @@
 #define	MSG_GC_HOLE		 120
 #define	MSG_GC_SWEEP		 121
 #define	MSG_GC_CHECK		 122
-#define	MSG_SHIFT_PROGRESS	 123
-#define	MSG_SHIFT_POINTER	 124
-#define	MSG_SHIFT_FRAME		 125
-#define	MSG_STACK_OVERFLOW	 126
+#define	MSG_SHIFT		 123
+#define	MSG_SHIFT_PROGRESS	 124
+#define	MSG_SHIFT_POINTER	 125
+#define	MSG_SHIFT_FRAME		 126
+#define	MSG_STACK_OVERFLOW	 127
 
 #define MSG_ATTVAR_LINK		 150
 #define MSG_CALL_RESIDUE_VARS	 151
@@ -168,6 +170,9 @@
 #define MSG_TRIE_PUT_TERM	 200
 #define MSG_TRIE_GC		 201
 
+#define MSG_ZIP			 250
+#define MSG_ZIP_STATE		 251
+
 #define MSG_TABLING_WORK	 300
 
 #define CHK_SECURE              1000
