@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2017, University of Amsterdam,
-                              VU University Amsterdam
+    Copyright (c)  1985-2018, University of Amsterdam,
+                              VU University Amsterdam,
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -395,7 +396,7 @@ COMMON(char *)		expandVars(const char *pattern, char *expanded, int len);
 COMMON(char *)		getwd(char *buf);
 COMMON(char *)		AbsoluteFile(const char *spec, char *path);
 COMMON(int)		IsAbsolutePath(const char *spec);
-COMMON(char *)		BaseName(const char *f);
+COMMON(char *)		BaseName(const char *f, char *buf);
 COMMON(char *)		DirName(const char *f, char *buf);
 COMMON(bool)		ChDir(const char *path);
 COMMON(atom_t)		TemporaryFile(const char *id,
@@ -613,9 +614,7 @@ COMMON(void)		resetSignals(void);
 COMMON(void)		cleanupSignals(void);
 COMMON(int)		handleSignals(ARG1_LD);
 
-COMMON(int)		initPrologStacks(size_t local,
-					 size_t global,
-					 size_t trail);
+COMMON(int)		initPrologStacks(size_t limit);
 COMMON(void)		initPrologLocalData(ARG1_LD);
 COMMON(void)		deallocateStacks(void);
 COMMON(bool)		restoreStack(Stack s);
@@ -625,6 +624,7 @@ COMMON(void)		freeStacks(ARG1_LD);
 COMMON(void)		freePrologLocalData(PL_local_data_t *ld);
 COMMON(int)		ensure_room_stack(Stack s, size_t n, int ex);
 COMMON(int)		trim_stack(Stack s);
+COMMON(int)		set_stack_limit(size_t limit);
 COMMON(void *)		stack_malloc(size_t size);
 COMMON(void *)		stack_realloc(void *old, size_t size);
 COMMON(void)		stack_free(void *mem);
@@ -679,12 +679,10 @@ COMMON(const char *)	atom_summary(atom_t name, unsigned int maxlen);
 COMMON(const char *)	string_summary(word name, unsigned int maxlen);
 
 /* pl-wic.c */
-COMMON(bool)		loadWicFromStream(IOSTREAM *fd);
+COMMON(bool)		loadWicFromStream(const char *rcpath, IOSTREAM *fd);
 COMMON(bool)		compileFileList(IOSTREAM *out, int argc, char **argv);
 COMMON(void)		qlfCleanup(void);
 
-COMMON(void)		wicPutNum(int64_t n, IOSTREAM *fd);
-COMMON(int64_t)		wicGetNum(IOSTREAM *fd);
 COMMON(void)		wicPutStringW(const pl_wchar_t *w, size_t len,
 				      IOSTREAM *fd);
 COMMON(pl_wchar_t*)	wicGetStringUTF8(IOSTREAM *fd, size_t *length,
@@ -745,21 +743,6 @@ COMMON(int)		ms_snprintf(char *buffer, size_t count,
 COMMON(void)		getDefaultsFromRegistry(void);
 COMMON(const char*)	WinError(void);
 
-/* pl-rc.c */
-COMMON(IOSTREAM *)       SopenRC(void *rca,
-			const char *name, const char *rcclass, int flags);
-COMMON(foreign_t)	pl_rc_handle(term_t h);
-COMMON(foreign_t)        pl_rc_open(term_t rc_h,
-			   term_t name, term_t class,
-			   term_t rw, term_t handle);
-COMMON(foreign_t)        pl_rc_open_archive(term_t file, term_t handle);
-COMMON(foreign_t)        pl_rc_close_archive(term_t rc_h);
-COMMON(foreign_t)        pl_rc_save_archive(term_t rc_h, term_t to);
-COMMON(foreign_t)        pl_rc_append_file(term_t rc_h,
-				  term_t name, term_t class, term_t encoding,
-				  term_t file);
-COMMON(foreign_t)	pl_rc_members(term_t rc_h, term_t members);
-
 /* pl-xterm.c */
 
 COMMON(foreign_t)	pl_open_xterm(term_t title,
@@ -804,5 +787,8 @@ COMMON(Code)	push_continuation(term_t cont, LocalFrame pfr, Code pcret
 				  ARG_LD);
 COMMON(Code)	shift(term_t ball ARG_LD);
 
+/* pl-variant.c */
+COMMON(int)	is_variant_ptr(Word p1, Word p2 ARG_LD);
+
 /* pl-version.h */
 COMMON(void)	setGITVersion(void);
