@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2017, University of Amsterdam,
+    Copyright (c)  1985-2018, University of Amsterdam,
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -1688,8 +1689,10 @@ struct sourceFile
   int		magic;			/* Magic number */
   int		count;			/* number of times loaded */
   unsigned	number_of_clauses;	/* number of clauses */
-  unsigned	index : 24;		/* index number (1,2,...) */
-  unsigned	system : 1;		/* system sourcefile: do not reload */
+  unsigned	index     : 24;		/* index number (1,2,...) */
+  unsigned	system     : 1;		/* system sourcefile: do not reload */
+  unsigned	from_state : 1;		/* Loaded from resource DB state */
+  unsigned	resource   : 1;		/* Loaded from resource DB file */
 };
 
 typedef struct srcfile_array
@@ -1947,7 +1950,6 @@ this to enlarge the runtime stacks.  Otherwise use the stack-shifter.
 	{ type		base;		/* base address of the stack */     \
 	  type		top;		/* current top of the stack */      \
 	  type		max;		/* allocated maximum */		    \
-	  size_t	size_limit;	/* Max size the stack can grow to */\
 	  size_t	gced_size;	/* size after last GC */	    \
 	  size_t	small;		/* Do not GC below this size */	    \
 	  size_t	spare;		/* Current reserved area */	    \
@@ -1962,10 +1964,9 @@ this to enlarge the runtime stacks.  Otherwise use the stack-shifter.
 
 struct stack STACK(caddress);		/* Anonymous stack */
 
-#define N_STACKS (4)
-
 typedef struct
-{ struct STACK(LocalFrame) local;	/* local (environment) stack */
+{ size_t limit;				/* Total stack limit */
+  struct STACK(LocalFrame) local;	/* local (environment) stack */
   struct STACK(Word)	   global;	/* local (environment) stack */
   struct STACK(TrailEntry) trail;	/* trail stack */
   struct STACK(Word *)	   argument;	/* argument stack */
@@ -2005,23 +2006,27 @@ typedef struct
 #define sizeStackP(s) ((intptr_t)((char *)(s)->max - (char *)(s)->base))
 #define roomStackP(s) ((intptr_t)((char *)(s)->max - (char *)(s)->top))
 #define spaceStackP(s) (limitStackP(s)-usedStackP(s))
-#define limitStackP(s) ((intptr_t)((s)->size_limit))
 #define narrowStackP(s) (roomStackP(s) < (intptr_t)(s)->minfree)
 
 #define usedStack(name) usedStackP(&LD->stacks.name)
 #define sizeStack(name) sizeStackP(&LD->stacks.name)
 #define roomStack(name) roomStackP(&LD->stacks.name)
 #define spaceStack(name) spaceStackP(&LD->stacks.name)
-#define limitStack(name) limitStackP(&LD->stacks.name)
 #define narrowStack(name) narrowStackP(&LD->stacks.name)
 
-#define GROW_TRIM ((size_t)-1)
+#define globalStackLimit() (LD->stacks.limit > (MAXTAGGEDPTR+1) ? \
+					       (MAXTAGGEDPTR+1) : \
+					       LD->stacks.limit)
+
+#define GROW_TRIM  ((size_t)-1)
+#define GROW_TIGHT ((size_t)1)
 
 #define	LOCAL_OVERFLOW	  (-1)
 #define	GLOBAL_OVERFLOW	  (-2)
 #define	TRAIL_OVERFLOW	  (-3)
 #define	ARGUMENT_OVERFLOW (-4)
-#define	MEMORY_OVERFLOW   (-5)		/* out of malloc()-heap */
+#define STACK_OVERFLOW    (-5)		/* total stack limit overflow */
+#define	MEMORY_OVERFLOW   (-6)		/* out of malloc()-heap */
 
 #define ALLOW_NOTHING	0x0
 #define ALLOW_GC	0x1		/* allow GC on stack overflow */
@@ -2055,6 +2060,7 @@ size N on the global stack AND  can   use  bindConst()  to bind it to an
 #define overflowCode(n) \
 	( (gTop+(n)+BIND_GLOBAL_SPACE > gMax) ? GLOBAL_OVERFLOW \
 					      : TRAIL_OVERFLOW )
+#define GLOBAL_TRAIL_RATIO (6)
 
 
 		 /*******************************
