@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2014-2017, VU University Amsterdam
+    Copyright (c)  2014-2018, VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -257,23 +258,23 @@ lookupSourceFile_unlocked(atom_t name, int create)
     GD->files.no_hole_before = 1;
   }
 
-  if ( (file=lookupHTable(GD->files.table, (void*)name)) )
-  { ;
-  } else if ( create )
+  if ( !(file=lookupHTable(GD->files.table, (void*)name)) &&
+       create )
   { file = allocHeapOrHalt(sizeof(*file));
     memset(file, 0, sizeof(*file));
-    file->name = name;
-    file->system = GD->bootsession;
+
+    file->name       = name;
+    file->system     = GD->bootsession;
+    file->from_state = GD->bootsession;
+    file->resource   = GD->bootsession;
 #ifdef O_PLMT
-    file->mutex = allocSimpleMutex(PL_atom_chars(name));
+    file->mutex    = allocSimpleMutex(PL_atom_chars(name));
 #endif
+    file->magic    = SF_MAGIC;
     PL_register_atom(file->name);
-    file->magic = SF_MAGIC;
     registerSourceFile(file);
 
     addNewHTable(GD->files.table, (void*)name, file);
-  } else
-  { file = NULL;
   }
 
   return file;
@@ -459,24 +460,6 @@ unlinkSourceFileModule(SourceFile sf, Module m)
 }
 
 
-static
-PRED_IMPL("$make_system_source_files", 0, make_system_source_files, 0)
-{ int i, n;
-
-  PL_LOCK(L_SRCFILE);
-  n = highSourceFileIndex();
-  for(i=1; i<n; i++)
-  { SourceFile f = indexToSourceFile(i);
-
-    if ( f )
-      f->system = TRUE;
-  }
-  PL_UNLOCK(L_SRCFILE);
-
-  return TRUE;
-}
-
-
 /** '$source_file'(:Head, -File) is semidet.
 */
 
@@ -603,10 +586,68 @@ PRED_IMPL("$source_file_property", 3, source_file_property, 0)
     return PL_unify_bool(A3, sf ? sf->reload != NULL : 0);
   if ( property == ATOM_number_of_clauses )
     return PL_unify_integer(A3, sf ? sf->number_of_clauses : 0);
+  if ( property == ATOM_resource )
+    return PL_unify_bool(A3, sf ? sf->resource : FALSE);
+  if ( property == ATOM_from_state )
+    return PL_unify_bool(A3, sf ? sf->from_state : FALSE);
 
   return PL_domain_error("source_file_property", A2);
 }
 
+static
+PRED_IMPL("$set_source_file", 3, set_source_file, 0)
+{ PRED_LD
+  atom_t filename, property;
+  SourceFile sf;
+
+  if ( !PL_get_atom_ex(A1, &filename) ||
+       !PL_get_atom_ex(A2, &property) )
+    return FALSE;
+
+  if ( (sf = lookupSourceFile(filename, FALSE)) )
+  { if ( property == ATOM_resource )
+    { int v;
+
+      if ( PL_get_bool_ex(A3, &v) )
+      { sf->resource = v;
+	return TRUE;
+      }
+      return FALSE;
+    } else
+      return PL_domain_error("source_file_property", A2);
+  } else
+    return PL_existence_error("source_file", A1);
+
+}
+
+static
+PRED_IMPL("$set_source_files", 1, set_source_files, 0)
+{ PRED_LD
+  atom_t prop;
+
+  if ( !PL_get_atom_ex(A1, &prop) )
+    return FALSE;
+  if ( prop == ATOM_system || prop == ATOM_from_state )
+  { int i, n;
+
+    PL_LOCK(L_SRCFILE);
+    n = highSourceFileIndex();
+    for(i=1; i<n; i++)
+    { SourceFile f = indexToSourceFile(i);
+
+      if ( f )
+      { if ( prop == ATOM_system )
+	  f->system = TRUE;
+	f->from_state = TRUE;
+      }
+    }
+    PL_UNLOCK(L_SRCFILE);
+
+    return TRUE;
+  } else
+    return PL_domain_error("source_property", A1);
+}
+
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 unloadFile(SourceFile sf)
@@ -1617,11 +1658,12 @@ BeginPredDefs(srcfile)
   PRED_DEF("$source_file_predicates",	2, source_file_predicates,   0)
   PRED_DEF("$time_source_file",		3, time_source_file,    PL_FA_NONDET)
   PRED_DEF("$source_file_property",	3, source_file_property,     0)
+  PRED_DEF("$set_source_file",          3, set_source_file,          0)
   PRED_DEF("$clause_from_source",	4, clause_from_source,	     0)
   PRED_DEF("$unload_file",		1, unload_file,		     0)
   PRED_DEF("$start_consult",		2, start_consult,	     0)
   PRED_DEF("$end_consult",		1, end_consult,		     0)
-  PRED_DEF("$make_system_source_files",	0, make_system_source_files, 0)
+  PRED_DEF("$set_source_files",	        1, set_source_files,	     0)
   PRED_DEF("$flush_predicate",		1, flush_predicate,          0)
   PRED_DEF("$flushed_predicate",	1, flushed_predicate,	     0)
 EndPredDefs
